package com.abi.task.common.excel;

import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.excel.common.AbstractExcelProperty;
import com.abi.task.common.excel.common.ExcelHeadAlias;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * excel导出工厂类
 * @Author AndyTan
 * @Date 2021/2/24
 */
@Slf4j
public class ExcelInputFactory {

    /**
     * 导入excel
     * @param file
     * @param t
     * @param <T>
     * @return
     */
    public static <T extends AbstractExcelProperty> List<T> getExcelPojoList(MultipartFile file, Class<T> t){
        //1-校验入参文件
        checkExcelFile(file);

        //2-file转map
        List<Map<String,Object>> mapList= importExcel(file,t);

        //3-map反射转pojoList
        List<T> pojoList = mapListToPojoList(mapList, t);

        return pojoList;
    }

    /**
     * 导出excel空模板
     * @param excelFileName
     * @param response
     * @param clz
     * @param <T>
     */
    public static <T extends AbstractExcelProperty> void downloadExcelModel(String excelFileName,
            HttpServletResponse response,Class<T> clz){
        //调用导出方法
        ExcelOutputFactory.outputStreamExcel(excelFileName,response,null,clz);
    }


    /**
     * 校验excel文件后缀
     * @param file
     */
    private static void checkExcelFile(MultipartFile file){
        if (file != null && file.getSize() > 0) {
            String fileName = file.getOriginalFilename();
            String suffix = fileName.substring(fileName.lastIndexOf(".") + 1);
            if(!"xlsx".equals(suffix)&&!"xls".equals(suffix)){
                throw new BusinessException("上传文件格式不正确");
            }
        } else {
            throw new BusinessException("上传文件不存在");
        }
    }

    /**
     * 读取excel表格内容返回List<Map>
     * @param file
     * @param t
     * @param <T>
     * @return
     */
    public static <T extends AbstractExcelProperty> List<Map<String,Object>> importExcel(MultipartFile file,Class<T> t){
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            log.error("读取文件失败", e);
        }

        ExcelReader reader = ExcelUtil.getReader(inputStream);
        List<Object> header=reader.readRow(0);

        //字段数组
        //表头数组 e.g. "单位名称","单位类型"
        List<String> headList = new ArrayList<>();
        //表头别名数组 e.g. "name","typeName"
        List<String> headerAliaList = new ArrayList<>();
        //循环获取模板的字段和别名数组
        Field[] feilds = t.getDeclaredFields();
        for(Field field:feilds){
            try {
                String fieldName = field.getName();
                ExcelHeadAlias headAlias = field.getAnnotation(ExcelHeadAlias.class);
                String fieldDesc = headAlias.value();
                //放入列表
                headList.add(fieldDesc);
                headerAliaList.add(fieldName);
            }catch (Exception e){
                log.info("toMapList error",e);
            }
        }

        //替换表头关键字
        if(header.size()==0){
            throw new BusinessException("导入文件内容样式错误，请参考下载模板填写");
        }
        for(int i=0;i<headList.size();i++){
            if(headList.get(i).equals(header.get(i))){
                reader.addHeaderAlias(headList.get(i),headerAliaList.get(i));
            }else{
                throw new BusinessException("导入文件内容样式错误，请参考下载模板填写");
            }
        }

        //读取excel，获得map
        //1：表头所在行数  2：数据开始读取位置   Integer.MAX_VALUE:数据读取结束行位置
        List<Map<String,Object>> excelContentMap = reader.read(0,0,Integer.MAX_VALUE);

        return excelContentMap;
    }

    private static <T extends AbstractExcelProperty> List<T> mapListToPojoList(List<Map<String,Object>> mapList,Class<T> tClass){
        List<T> excelPojoList = new ArrayList<>();
        for(Map<String,Object> mapTemp:mapList){
            try {
                T t = tClass.newInstance();
                //方法map缓存
                Map<String,Method> methodCache = new HashMap<>();
                //循环每一个字段，反射放入数据
                for(Map.Entry<String, Object> entry:mapTemp.entrySet()){
                    String fieldName = entry.getKey();
                    Object value = entry.getValue();

                    //获取method
                    if (!methodCache.containsKey(fieldName)) {
                        String methodName = "set" + fieldName.substring(0, 1).toUpperCase() + fieldName.substring(1);
                        Method[] methods = tClass.getMethods();
                        for(Method method:methods){
                            if(methodName.equals(method.getName())){
                                methodCache.put(fieldName, method);
                                break;
                            }

                        }
                    }
                    //调用method
                    if(value!=null){
                        Method method = methodCache.get(fieldName);
                        method.invoke(t,String.valueOf(value));
                    }
                }
                excelPojoList.add(t);
            }catch(Exception e){
                log.error("mapListToPojoList error",e);
            }
        }

        return excelPojoList;
    }



}
