package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.BaseDepartment;
import com.abi.qms.platform.dao.entity.BaseFactory;
import com.abi.qms.platform.dao.enums.FactoryTypeEnum;
import com.abi.qms.platform.dao.enums.ValidEnum;
import com.abi.qms.platform.dao.mapper.BaseFactoryMapper;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.ExcelImportRes;
import com.abi.qms.platform.dto.res.GetFactoryDetailRes;
import com.abi.qms.platform.dto.res.ListDeptRes;
import com.abi.qms.platform.dto.res.ListFactoryRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.FactoryService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.excel.preperties.DepartmentProperty;
import com.abi.task.common.excel.preperties.FactoryProperty;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 供应商管理
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class FactoryServiceImpl implements FactoryService {

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    /**
     * 供应商新增修改
     **/
    @Override
    public void saveFactory(SaveFactoryReq req) {

        //0-校验入参
        //code不能重复
        QueryWrapper<BaseFactory> factoryQw = new QueryWrapper<>();
        factoryQw.eq("factory_code", req.getFactoryCode());
        if (req.getId() != null) {
            factoryQw.ne("id", req.getId());
        }
        List<BaseFactory> checkFactoryList = baseFactoryMapper.selectList(factoryQw);
        if (CollectionUtil.isNotEmpty(checkFactoryList)) {
            throw new BusinessException("供应商ID已存在");
        }

        //1-声明对象
        BaseFactory dept = null;
        if (req.getId() != null) {
            dept = baseFactoryMapper.selectById(req.getId());
            AssertUtil.isNull(dept, "部门不存在");
        } else {
            dept = new BaseFactory();
        }

        //2-放入数据
        dept.setFactoryCode(req.getFactoryCode());
        dept.setFactoryName(req.getFactoryName());
        dept.setFactoryType(req.getFactoryType());
        dept.setKeymanName(req.getKeymanName());
        dept.setKeymanMobile(req.getKeymanMobile());
        dept.setEmail(req.getEmail());
        dept.setMaxFileCount(req.getMaxFileCount());
        dept.setDescription(req.getDescription());

        //3-新增or修改
        if (req.getId() != null) {
            baseFactoryMapper.updateById(dept);
        } else {
            baseFactoryMapper.insert(dept);
        }
        
    }

    /**
     * 供应商查询分页
     **/
    @Override
    public ListFactoryRes listFactory(ListFactoryReq req) {
        //1-拼接条件
        QueryWrapper<BaseFactory> factoryQw = new QueryWrapper<>();
        factoryQw.like(StringUtils.isNotBlank(req.getFactoryName()), "factory_name", "%" + req.getFactoryName() + "%");
        factoryQw.eq(req.getFactoryType() != null, "factory_type", req.getFactoryType());
        factoryQw.eq(req.getValid() != null, "valid", req.getValid());

        //2-分页查询
        IPage<BaseFactory> iPage = baseFactoryMapper.selectPage(PageUtil.createPage(req), factoryQw);
        List<BaseFactory> factoryList = iPage.getRecords();

        //封装出参
        ListFactoryRes res = new ListFactoryRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListFactoryRes.FactoryBean> factoryBeanList = PojoConverterUtils.copyList(factoryList, ListFactoryRes.FactoryBean.class);
        res.setFactoryBeanList(factoryBeanList);

        return res;
    }

    /**
     * 供应商查询详情
     **/
    @Override
    public GetFactoryDetailRes getFactoryDetail(GetFactoryDetailReq req) {
        //1-查询供应商对象
        BaseFactory factory = baseFactoryMapper.selectById(req.getId());
        AssertUtil.isNull(factory,"供应商不存在");

        //转化出参
        GetFactoryDetailRes res = PojoConverterUtils.copy(factory, GetFactoryDetailRes.class);

        return res;
    }

    /**
     * 供应商启用
     **/
    @Override
    public void enableFactory(EnableFactoryReq req) {
        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BaseFactory factory = baseFactoryMapper.selectById(id);
            AssertUtil.isNull(factory, "供应商不存在");
            if (!ValidEnum.NOT_VALID.is(factory.getValid())) {
                continue;
            }

            //2-修改状态
            factory.setValid(ValidEnum.VALID.getCode());
            baseFactoryMapper.updateById(factory);
        }
    }

    /**
     * 供应商禁用
     **/
    @Override
    public void disableFactory(DisableFactoryReq req) {
        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BaseFactory factory = baseFactoryMapper.selectById(id);
            AssertUtil.isNull(factory, "部门不存在");
            if (!ValidEnum.VALID.is(factory.getValid())) {
                return;
            }

            //2-修改状态
            factory.setValid(ValidEnum.NOT_VALID.getCode());
            baseFactoryMapper.updateById(factory);
        }
    }

    /**
     * excel批量导入
     **/
    @Override
    public ExcelImportRes factoryExcelImport(List<FactoryProperty> factoryPropertyList) {
        List<ExcelImportRes.FailBean> failList = new ArrayList<>();

        //1-循环导入，有异常则捕获
        for(int i=0;i<factoryPropertyList.size();i++){
            FactoryProperty factoryProperty = factoryPropertyList.get(i);
            try{
                //1-校验必填与枚举
                String errorMessage = checkFactoryProperty(factoryProperty);
                if(StringUtils.isNotBlank(errorMessage)){
                    throw new BusinessException(errorMessage);
                }

                //2-调用业务代码新增
                SaveFactoryReq req = new SaveFactoryReq();
                req.setFactoryCode(factoryProperty.getFactoryCode());
                req.setFactoryName(factoryProperty.getFactoryName());
                req.setFactoryType(FactoryTypeEnum.getCode(factoryProperty.getFactoryType()));
                req.setKeymanName(factoryProperty.getKeymanName());
                req.setKeymanMobile(factoryProperty.getKeymanMobile());
                req.setEmail(factoryProperty.getEmail());
                req.setMaxFileCount(Long.valueOf(factoryProperty.getMaxFileCount()));
                req.setDescription(factoryProperty.getDescription());

                saveFactory(req);
            }catch (BusinessException e){
                failList.add(new ExcelImportRes.FailBean(i+2,e.getMessage()));
            }
        }

        //2-返回导入结果
        ExcelImportRes res = new ExcelImportRes();
        res.setFailedCount(failList.size());
        res.setSuccessCount(factoryPropertyList.size()-res.getFailedCount());
        res.setFailBeanList(failList);

        return res;
        
    }

    /**
     * 检查批量导入的内容
     * @param factoryProperty
     * @return
     */
    private String checkFactoryProperty(FactoryProperty factoryProperty){
        StringBuffer errorMessage = new StringBuffer();
        //逐个字段校验
        if(StringUtils.isBlank(factoryProperty.getFactoryCode())){
            errorMessage.append("供应商id").append("为空,");
        }
        if(StringUtils.isBlank(factoryProperty.getFactoryName())){
            errorMessage.append("供应商名称").append("为空,");
        }
        if(StringUtils.isBlank(factoryProperty.getFactoryType())){
            errorMessage.append("供应商类型").append("为空,");
        }
        if(StringUtils.isNotBlank(factoryProperty.getFactoryType())
                &&FactoryTypeEnum.getCode(factoryProperty.getFactoryType())==null){
            errorMessage.append("供应商类型").append("枚举值不正确,");
        }
        if(StringUtils.isBlank(factoryProperty.getKeymanName())){
            errorMessage.append("联系人姓名").append("为空,");
        }
        if(StringUtils.isBlank(factoryProperty.getKeymanMobile())){
            errorMessage.append("联系人手机号").append("为空,");
        }
        if(StringUtils.isBlank(factoryProperty.getEmail())){
            errorMessage.append("邮箱").append("为空,");
        }

        return errorMessage.toString();
    }


}
